@@ -351,10 +351,14 @@ void CPlusGenerator::generateMethodCode()
     fout << "// File: "<< _parserName << ".cc\n";
     fout << "// This code was produced by the dragon parser generator\n";
     fout << "//\n";
-
+    
     fout << "#include \"" << _parserName << ".h\"\n";
     fout << "#include <lfcbase/Exception.h>\n";
     fout << "#include <lfcbase/StackT.h>\n";
+    
+    // we need stdlib.h for malloc 
+    if ( _dynamicTable )
+	fout << "#include <stdlib.h>\n";
 
     fout << "\n";
     fout << _parserName << "::ScannerStateEntry::ScannerStateEntry()\n";
@@ -902,7 +906,7 @@ void CPlusGenerator::generateMethodCode()
 
     fout << "               return;\n";
     fout << "            }\n";
-    fout << "            default:\n";
+    fout << "            case NULLACTION:\n";
     fout << "               break;\n";
     fout << "         }\n";
     fout << "      }\n";
@@ -1005,6 +1009,10 @@ void CPlusGenerator::generateMethodCode()
     fout << "      {\n";
     fout << "         _tokenVal[i]=c;\n";
     fout << "         i++;\n";
+    fout << "         if ( i >= MAXTOKENVAL )\n";
+    fout << "         {\n";
+    fout << "            throw Exception(EXLOC, Chain(\"Token length exceeded\"));\n";
+    fout << "         }\n";
     fout << "         c = nextChar();\n";
     fout << "      }\n"; 
     fout << "      if (isSepSign(c)) backChar();\n";
